'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.default = defaultResolver;

function _path() {
  const data = require('path');

  _path = function () {
    return data;
  };

  return data;
}

function _jestPnpResolver() {
  const data = _interopRequireDefault(require('jest-pnp-resolver'));

  _jestPnpResolver = function () {
    return data;
  };

  return data;
}

function _resolve() {
  const data = require('resolve');

  _resolve = function () {
    return data;
  };

  return data;
}

var _resolve2 = require('resolve.exports');

function _slash() {
  const data = _interopRequireDefault(require('slash'));

  _slash = function () {
    return data;
  };

  return data;
}

var _fileWalkers = require('./fileWalkers');

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
function defaultResolver(path, options) {
  // Yarn 2 adds support to `resolve` automatically so the pnpResolver is only
  // needed for Yarn 1 which implements version 1 of the pnp spec
  if (process.versions.pnp === '1') {
    return (0, _jestPnpResolver().default)(path, options);
  }

  const result = (0, _resolve().sync)(path, {
    ...options,
    isDirectory: _fileWalkers.isDirectory,
    isFile: _fileWalkers.isFile,
    packageFilter: createPackageFilter(path, options.packageFilter),
    pathFilter: createPathFilter(path, options.conditions, options.pathFilter),
    preserveSymlinks: false,
    readPackageSync,
    realpathSync: _fileWalkers.realpathSync
  }); // Dereference symlinks to ensure we don't create a separate
  // module instance depending on how it was referenced.

  return (0, _fileWalkers.realpathSync)(result);
}
/*
 * helper functions
 */

function readPackageSync(_, file) {
  return (0, _fileWalkers.readPackageCached)(file);
}

function createPackageFilter(originalPath, userFilter) {
  if (shouldIgnoreRequestForExports(originalPath)) {
    return userFilter;
  }

  return function packageFilter(pkg, ...rest) {
    let filteredPkg = pkg;

    if (userFilter) {
      filteredPkg = userFilter(filteredPkg, ...rest);
    }

    if (filteredPkg.exports == null) {
      return filteredPkg;
    }

    return {
      ...filteredPkg,
      // remove `main` so `resolve` doesn't look at it and confuse the `.`
      // loading in `pathFilter`
      main: undefined
    };
  };
}

function createPathFilter(originalPath, conditions, userFilter) {
  if (shouldIgnoreRequestForExports(originalPath)) {
    return userFilter;
  }

  const options = conditions
    ? {
        conditions,
        unsafe: true
      } // no conditions were passed - let's assume this is Jest internal and it should be `require`
    : {
        browser: false,
        require: true
      };
  return function pathFilter(pkg, path, relativePath, ...rest) {
    let pathToUse = relativePath;

    if (userFilter) {
      pathToUse = userFilter(pkg, path, relativePath, ...rest);
    }

    if (pkg.exports == null) {
      return pathToUse;
    } // this `index` thing can backfire, but `resolve` adds it: https://github.com/browserify/resolve/blob/f1b51848ecb7f56f77bfb823511d032489a13eab/lib/sync.js#L192

    const isRootRequire =
      pathToUse === 'index' && !originalPath.endsWith('/index');
    const newPath = isRootRequire ? '.' : (0, _slash().default)(pathToUse);
    return (0, _resolve2.resolve)(pkg, newPath, options) || pathToUse;
  };
} // if it's a relative import or an absolute path, exports are ignored

const shouldIgnoreRequestForExports = path =>
  path.startsWith('.') || (0, _path().isAbsolute)(path);
